#!/usr/bin/env python

import sys
import PAM
from getpass import getpass

def pam_conv(auth, query_list):

    resp = []
    
    for i in range(len(query_list)):
        query, type = query_list[i]
        if type == PAM.PAM_PROMPT_ECHO_ON:
            val = raw_input(query)
            resp.append((val, 0))
        elif type == PAM.PAM_PROMPT_ECHO_OFF:
            val = getpass(query)
            resp.append((val, 0))
        elif type == PAM.PAM_PROMPT_ERROR_MSG or type == PAM.PAM_PROMPT_TEXT_INFO:
            print query
            resp.append(('', 0));
        else:
            return None

    return resp

service = 'passwd'

if len(sys.argv) == 2:
    user = sys.argv[1]
else:
    user = None

auth = PAM.pam()
auth.start(service)

print auth

if user != None:
    auth.set_item(PAM.PAM_USER, user)

auth.set_item(PAM.PAM_CONV, pam_conv)

auth.set_userdata("testing user data")

try:
    auth.authenticate()
    auth.acct_mgmt()
except PAM.error, (resp, code):
    print 'Go away! (%s)' % resp
except:
    print 'Internal error'
else:
    print 'Good to go!'



#!/usr/bin/env python

import sys
import unittest
import PAM

class PAMConstructorTestCase(unittest.TestCase):

    def testConstructor(self):
        """The constructor works with no arguements and fails with
        a TypeError when called with any arguments."""
        
        pam = PAM.pam()
        self.assertRaises(TypeError, PAM.pam, None)
        self.assertRaises(TypeError, PAM.pam, 1)
        self.assertRaises(TypeError, PAM.pam, "")
        self.assertRaises(TypeError, PAM.pam, ())
        self.assertRaises(TypeError, PAM.pam, [])
        self.assertRaises(TypeError, PAM.pam, {})


class PAMTestCase(unittest.TestCase):

    def setUp(self):
    
        self.pam = PAM.pam()

    def test_get_userdata_interface(self):
        """get_userdata() raises TypeError when called with any arguments."""

        self.assertRaises(TypeError, self.pam.get_userdata, 1)

    def test_set_userdata_interface(self):
        """set_userdata() raises TypeError when called with 0 arguments
        or with more than 1 argument."""
        
        self.assertRaises(TypeError, self.pam.set_userdata)
        self.assertRaises(TypeError, self.pam.set_userdata, 1, 2)


    def test_userdata_default(self):
        """The default value for userdata is None."""
    
        self.failUnless(self.pam.get_userdata() is None)

    def test_userdata(self):
        """The userdata getter and setter will store and return any data."""
    
        self.pam.set_userdata(1)
        self.failUnless(self.pam.get_userdata() == 1)

    def test_start(self):
        """pam.start() works as expected."""
        
        self.assertRaises(TypeError, self.pam.start)
        self.assertRaises(TypeError, self.pam.start, 1)
        

if __name__ == '__main__':

    unittest.main()



#!/usr/bin/env python

from distutils.core import setup, Extension

setup(
    name='PyPAM',
    version='0.5.0',
    description='PAM (Pluggable Authentication Module) bindings for Python',
    author='Rob Riggs',
    author_email='rob+pypam@pangalactic.org',
    url='http://www.pangalactic.org/PyPAM',
    license='LGPL',
    ext_modules=[
        Extension(
            'PAMmodule',
            ['PAMmodule.c'],
            libraries=['pam', 'pam_misc'],
            extra_compile_args = ['-std=c99'],
        )
    ],
)

